package com.perforce.p4simulink.connection;

/**
 * Perforce SCM Integration
 */

import com.perforce.p4java.PropertyDefs;
import com.perforce.p4java.client.IClient;
import com.perforce.p4java.impl.mapbased.rpc.RpcPropertyDefs;
import com.perforce.p4java.server.IOptionsServer;
import com.perforce.p4java.server.ServerFactory;
import com.perforce.p4simulink.P4CMException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.Closeable;
import java.io.IOException;
import java.util.Properties;

public class Connection implements Closeable {
    private static final Properties properties;
    private static final Logger log = LogManager.getLogger(Connection.class.getName());

    IOptionsServer server;
    IClient client;

    /**
     * Initialise static properties for all Perforce connections	*/
    static {
        properties = System.getProperties();
        properties.put(PropertyDefs.PROG_NAME_KEY, "P4Simulink");
        properties.put(PropertyDefs.PROG_VERSION_KEY, "2013.2");

        // Set up socket pooling to use a single socket
        properties.put(RpcPropertyDefs.RPC_SOCKET_POOL_SIZE_NICK, "1");
        // disable timeout for slow servers / large db lock times
        properties.put(RpcPropertyDefs.RPC_SOCKET_SO_TIMEOUT_NICK, "0");
    }

    /**
     * Connect to the Perforce Server
     *
     * @param config Perforce connection config (e,g. P4PORT, P4USER, ...)
     * @throws com.perforce.p4simulink.P4CMException
     */
    public void connect(Config config) throws P4CMException {
        try {
            if (server != null && server.isConnected()) {
                server.disconnect();
            }
            server = ServerFactory.getOptionsServer(config.getAddress(), properties);
            server.connect();
            server.setUserName(config.getUser());
            client = server.getClient(config.getClient());
            server.setCurrentClient(client);
        } catch (Exception e) {
            P4CMException ex = new P4CMException(e);
            log.error(ex);
            log.error(e.getMessage());
            throw ex;
        }
    }

    /**
     * Disconnect from the Perforce Server, if we're connected.
     *
     * @throws com.perforce.p4simulink.P4CMException
     */
    public void disconnect() throws P4CMException {
        try {
            if ((server != null) && server.isConnected()) {
                server.disconnect();
            }
        } catch (Exception e) {
            P4CMException ex = new P4CMException(e);
            log.error(ex);
            throw ex;
        }
    }

    @Override
    public void close() throws IOException {
        try {
            disconnect();
        } catch (P4CMException e) {
            throw new IOException(e);
        }
    }

    public boolean isReady() {
        if (server != null && server.isConnected()) {
            return true;
        }
        return false;
    }

    public IOptionsServer getServer() {
        return server;
    }
}
